#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Script para descargar archivos desde goear.
# Versión: 0.3
# Copyright (C) 2007 Daniel Fuentes <dbfuentes@gmail.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


# Importamos algunos módulos que luego usaremos en el script:

import sys
import urllib
import urllib2
from xml.dom import minidom

# El texto que se muestra en la ayuda

texto_ayuda = """Script para descargar archivos de goear
Modo de uso:
goear.py [Opción] [Audio_Url] 

Donde [Audio_Url] es la dirección (o url) del archivo, la cual es de la
siguiente forma http://www.goear.com/listen.php?v=xxxxx

Y en donde en [Opción] se puede ingresar una de las siguientes opciones:
siguientes opciones

Opciones disponibles:
-h, --help          Muestra la ayuda
-v, --version       Muestra la versión del programa
-n, --no-vervose    Omite el progreso de la descarga

Ejemplos:
goear.py http://www.goear.com/listen.php?v=xxxxx    # Descarga el archivo desde esa url
goear.py -n http://www.goear.com/listen.php?v=xxxxx # Descarga sin mostrar el progreso
goear.py --help                                     # Muestra esta ayuda

Para mayor información revise el archivo "README" """

# Algunas variables a usar:

progreso = True
opciones_ingresadas = []


# Comprobar a continuación si se ingreso algun dato en la linea de ordenes
# Si no se ingresa nada sys.argv solo tendra el nombre del script (goear.py)

if len(sys.argv) == 1: # El unico dato es goear.py (no hay ordenes o url)
    tengo_opciones = False
    audio_url = str(raw_input("Ingrese la dirección (url) del archivo: "))

elif len(sys.argv) == 2: 
# Dos casos posibles: se ingreso una opcion o la url (dirección)
# Además si la opcion es "-n" (caso 1) hay que preguntar la url 

    if sys.argv[1] == "-n" or sys.argv[1] == "--no-verbose":
        tengo_opciones = True
        opciones_ingresadas.append(sys.argv[1])
        audio_url = str(raw_input("Ingrese la dirección (url) del archivo: "))
    elif sys.argv[1] == "-h" or sys.argv[1] == "--help" or sys.argv[1] == "help" or sys.argv[1] == "-v" or sys.argv[1] == "--version":
        tengo_opciones = True
        opciones_ingresadas.append(sys.argv[1])
        audio_url = None
    elif sys.argv[1] == "moo":
        print """No hay ninguna Súper Vaca en este programa:
      ..::''''::..
    .;''        ``;.
   ::    ::  ::    ::
  ::     ::  ::     ::
  :: .:' ::  :: `:. ::
  ::  :          :  ::
   :: `:.      .:' ::
    `;..``::::''..;'
      ``::,,,,::'' """
        sys.exit()
    else:
        tengo_opciones = False
        audio_url = sys.argv[1]

elif len(sys.argv) > 2:
# Casos posibles: tener solo ordenes o tener opciones (ordenes) + dirección
    tengo_opciones = True
    for n in sys.argv[1:len(sys.argv)]:
        if n == "-n" or n == "--no-verbose" or n == "-h" or n== "--help" or n == "help" or n== "-v" or n== "--version":
            opciones_ingresadas.append(n)
            audio_url = None
        else:
            audio_url = n

# Verificar Opciones de la linea de comandos
# Y proceder de acuerdo a ellas

if tengo_opciones:
    for opcion in opciones_ingresadas:
        if opcion == "-h" or opcion == "--help" or opcion == "help":
            print texto_ayuda                    
            sys.exit()
        elif opcion == "-v" or opcion == "--version":
            print "Versión 0.3"
            sys.exit()
        elif opcion == "-n" or opcion == "--no-verbose":
            progreso = False

# Comprobar la dirección (url) del audio:
# Puede darse casos como el que sigue: goear.py -n -n
# En este caso aun no se ingresa la direción (pero el usuario no debería ingresar eso)

if audio_url == None:
    audio_url = str(raw_input("Ingrese la dirección (url) del archivo: "))

# Encontrar la ID del archivo

inicio_id = audio_url.find("v=")
fin_id = len(audio_url)

audio_id = audio_url[inicio_id+2:fin_id]

# Abrir el archivo xml que contiene la información 
# de la forma http://www.goear.com/files/sst/secmxxyyzz.xml

url_xml = "http://www.goear.com/files/sst/secm" + audio_id + ".xml"
data = None

try:
    xmldoc = urllib2.urlopen(url_xml,data)
    xml_data = minidom.parse(xmldoc)
    xmldoc.close()
    data = xml_data.toxml()
except urllib2.HTTPError: # Mensaje en caso de error
    print "Error, no se encuentra la información necesaria \n\nEsto puede ser provocado por ingresar mal la dirección (Url) del archivo o\ningresar una opción no valida. Por favor revise la ayuda con --help"
    sys.exit()

# Buscar la linea "song path=" que es la dirección del archivo

inicio_path = data.find("path=")
fin_path = data.find(".mp3")

path = data[inicio_path+6:fin_path+4]

# Definimos la función de descarga:

def download_file(filename, URL):
    global progreso # llamada a la variable progreso
    if progreso == False:
        f = urllib.urlretrieve(URL, filename)
    else:
        f = urllib.urlretrieve(URL, filename, reporthook=my_report_hook)

# Definimos la función que muestra el progreso del archivo:

def my_report_hook (block_count, block_size, total_size): 
    total_kb  = total_size / 1024
    descargado = block_count * (block_size/1024)
    print "Descargados %d kb de %d kb" %(descargado,total_kb)

# Ahora hay que descargar el archivo:

if __name__ == "__main__":
   download_file("musica.mp3",path)

print "Listo, ahora solo abra el archivo llamado musica.mp3"
